#property copyright "Copyright 2012, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <MovingAverages.mqh>

#property indicator_separate_window        //    
#property indicator_buffers 5              //  
#property indicator_plots 1                //   
#property indicator_type1 DRAW_LINE        //  
#property indicator_color1 Red             //  
#property indicator_width1 1               //  
#property indicator_label1 ""        // 

//  
enum Use_Symbol
  {
   Use_EURUSD = 1,                         // EURUSD
   Use_GBPUSD = 2,                         // GBPUSD
  }; 

//    
enum Use_Method
  {
   Use_SMA = 1,                            //  
   Use_EMA = 2,                            //  
   Use_SMMA = 3,                           //  
   Use_LWMA = 4,                           // - 
  };

//    
enum Use_Price
  {
   Use_CLOSE = 1,                          //  
   Use_OPEN = 2,                           //  
   Use_HIGH = 3,                           //    
   Use_LOW = 4,                            //    
   Use_MEDIAN = 5,                         //  , (high+low)/2
   Use_TYPICAL = 6,                        //  , (high+low+close)/3
   Use_WEIGHTED = 7,                       //  , (high+low+close+close)/4
  };

//      
input Use_Symbol MA_Symbol = Use_EURUSD;  //   
input bool MA_Reverse = false;            //  
input int MA_Fast = 8;                    //    
input int MA_Slow = 21;                   //    
input Use_Method MA_Method = Use_SMMA;    //  
input Use_Price MA_Price = Use_WEIGHTED;  //  

//  
double ExtMacdBuffer[];
double ExtFastMaBuffer[];
double ExtSlowMaBuffer[];
double ExtFastResMaBuffer[];
double ExtSlowResMaBuffer[];

// 
int ExtFastMaHandle;
int ExtSlowMaHandle;

int OnInit()
  {
  
   //    
   SetIndexBuffer(0, ExtMacdBuffer, INDICATOR_DATA);
   SetIndexBuffer(1, ExtFastMaBuffer, INDICATOR_CALCULATIONS);
   SetIndexBuffer(2, ExtSlowMaBuffer, INDICATOR_CALCULATIONS);
   SetIndexBuffer(3, ExtFastResMaBuffer, INDICATOR_CALCULATIONS);
   SetIndexBuffer(4, ExtSlowResMaBuffer, INDICATOR_CALCULATIONS);

   //  
   ExtFastMaHandle = iMA(Symbol_Name(MA_Symbol), 0, MA_Fast, 0, Method_Name(MA_Method), Price_Name(MA_Price));
   ExtSlowMaHandle = iMA(Symbol_Name(MA_Symbol), 0, MA_Slow, 0, Method_Name(MA_Method), Price_Name(MA_Price));

   //      
   bool Err = false;
   Err = (ExtFastMaHandle == INVALID_HANDLE || ExtSlowMaHandle == INVALID_HANDLE);
   if (Err)
     {
      Alert("  ,  ");
      return(-1);
     }      
   
   Print("  ");
   return(0);
  }

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
  
   //       
   int calculated = BarsCalculated(ExtFastMaHandle);
   if (calculated < rates_total)
     {
      Print("Not all data of ExtFastMaHandle is calculated (", calculated, "bars ). Error", GetLastError());
      return(0);
     } 
     
   //         
   calculated = BarsCalculated(ExtSlowMaHandle);
   if (calculated < rates_total)
     {
      Print("Not all data of ExtSlowMaHandle is calculated (", calculated, "bars ). Error", GetLastError());
      return(0);
     }     

   for (int b = 0; b < rates_total; b++) {ExtFastMaBuffer[b] = 0; ExtSlowMaBuffer[b] = 0;}
   
   //     
   if (CopyBuffer(ExtFastMaHandle, 0, 0, rates_total, ExtFastMaBuffer) <= 0)
     {
      Print("Getting fast MA is failed! Error", GetLastError());
      return(0);
     }
     
   //     
   if (CopyBuffer(ExtSlowMaHandle, 0, 0, rates_total, ExtSlowMaBuffer) <= 0)
     {
      Print("Getting slow MA is failed! Error", GetLastError());
      return(0);
     }       

   //  
   int weightsum = 1; 
   switch (MA_Method)
     {
      case 1:
         SimpleMAOnBuffer(rates_total, prev_calculated, 0, MA_Fast, ExtFastMaBuffer, ExtFastResMaBuffer);
         SimpleMAOnBuffer(rates_total, prev_calculated, 0, MA_Slow, ExtSlowMaBuffer, ExtSlowResMaBuffer);
         break;     
      case 2:
         ExponentialMAOnBuffer(rates_total, prev_calculated, 0, MA_Fast, ExtFastMaBuffer, ExtFastResMaBuffer);
         ExponentialMAOnBuffer(rates_total, prev_calculated, 0, MA_Slow, ExtSlowMaBuffer, ExtSlowResMaBuffer);
         break;
      case 3:
         SmoothedMAOnBuffer(rates_total, prev_calculated, 0, MA_Fast, ExtFastMaBuffer, ExtFastResMaBuffer);
         SmoothedMAOnBuffer(rates_total, prev_calculated, 0, MA_Slow, ExtSlowMaBuffer, ExtSlowResMaBuffer);
         break; 
      case 4:
         LinearWeightedMAOnBuffer(rates_total, prev_calculated, 0, MA_Fast, ExtFastMaBuffer, ExtFastResMaBuffer, weightsum);
         LinearWeightedMAOnBuffer(rates_total, prev_calculated, 0, MA_Slow, ExtSlowMaBuffer, ExtSlowResMaBuffer, weightsum);
         break;                  
     } 
   
   for(int i = prev_calculated; i < rates_total; i++)
     {
      ExtMacdBuffer[i] = ExtFastResMaBuffer[i] - ExtSlowResMaBuffer[i];
     }
   
   return(rates_total);
  }

void OnTradeTransaction(const MqlTradeTransaction& trans,
                        const MqlTradeRequest& request,
                        const MqlTradeResult& result)
  {
   
  }

//    
string Symbol_Name(int Symbol_Int)
  {
   string Symbol_String;
   if (Symbol_Int == 1) Symbol_String = "EURUSD";
   if (Symbol_Int == 2) Symbol_String = "GBPUSD";
   return(Symbol_String);
  }
  
//       
ENUM_MA_METHOD Method_Name(int Method_Int)
  {
   ENUM_MA_METHOD Method_Enum;
   if (Method_Int == 1) Method_Enum = MODE_SMA;
   if (Method_Int == 2) Method_Enum = MODE_EMA;
   if (Method_Int == 3) Method_Enum = MODE_SMMA;
   if (Method_Int == 4) Method_Enum = MODE_LWMA;
   return(Method_Enum);
  }  

//      
ENUM_APPLIED_PRICE Price_Name(int Price_Int)
  {
   ENUM_APPLIED_PRICE Price_Enum;
   if (Price_Int == 1) Price_Enum = PRICE_CLOSE;
   if (Price_Int == 2) Price_Enum = PRICE_OPEN;
   if (Price_Int == 3) Price_Enum = PRICE_HIGH;
   if (Price_Int == 4) Price_Enum = PRICE_LOW;
   if (Price_Int == 5) Price_Enum = PRICE_MEDIAN;
   if (Price_Int == 6) Price_Enum = PRICE_TYPICAL;
   if (Price_Int == 7) Price_Enum = PRICE_WEIGHTED;
   return(Price_Enum);
  }  